from microservice_template_core.tools.logger import get_logger
from microservice_template_core.settings import KafkaConfig
from kafka import KafkaProducer
import json
import traceback
from prometheus_client import Summary

logger = get_logger()


class KafkaProduceMessages(object):
    KAFKA_PRODUCER_START = Summary('kafka_producer_start_latency_seconds', 'Time spent starting Kafka producer')
    KAFKA_PRODUCE_MESSAGES = Summary('kafka_produce_messages_latency_seconds', 'Time spent processing produce to Kafka')

    def __init__(self):
        self.producer = self.init_producer()

    @staticmethod
    @KAFKA_PRODUCER_START.time()
    def init_producer():
        try:
            producer = KafkaProducer(
                value_serializer=lambda v: json.dumps(v).encode('utf-8'),
                bootstrap_servers=KafkaConfig.KAFKA_SERVERS
            )

            return producer
        except Exception as err:
            logger.error(
                msg=f"Can`t connect to Kafka cluster - {KafkaConfig.KAFKA_SERVERS}\nError: {err}\nTrace: {traceback.format_exc()}"
            )
            return None

    @KAFKA_PRODUCE_MESSAGES.time()
    def produce_message(self, topic, message):
        logger.debug(
            msg=f"Start producing message to topic - {topic}\nBody: {message}"
        )
        try:
            self.producer.send(topic, message)
            logger.debug(
                msg=f"Successfully pushed message to - {topic}"
            )
        except Exception as err:
            logger.error(
                msg=f"Can`t push message to - {topic}\nBody: {message}\nError: {err}\nTrace: {traceback.format_exc()}"
            )



import aerospike
from microservice_template_core.tools.logger import get_logger
from microservice_template_core.settings import AerospikeConfig
from prometheus_client import Summary

logger = get_logger()


class AerospikeClient(object):
    # Prometheus Metrics
    AEROSPIKE_CONNECTIONS = Summary('aerospike_connections_latency_seconds', 'Time spent processing connect to aerospike')
    AEROSPIKE_READ = Summary('aerospike_read_latency_seconds', 'Time spent processing read from aerospike')
    AEROSPIKE_WRITE = Summary('aerospike_write_latency_seconds', 'Time spent processing write to aerospike')
    AEROSPIKE_SCAN = Summary('aerospike_scan_latency_seconds', 'Time spent processing scan in aerospike')

    def __init__(self):
        self.client = self.client_aerospike()

    @staticmethod
    @AEROSPIKE_CONNECTIONS.time()
    def client_aerospike():
        config = {
            'hosts': [(AerospikeConfig.AEROSPIKE_HOST, AerospikeConfig.AEROSPIKE_PORT)]
        }

        try:
            client = aerospike.client(config).connect()
            logger.info(msg=f"Connected to Aerospike - {config['hosts']}")
        except Exception as err:
            client = None
            logger.error(msg=f"failed to connect to the cluster with: {err} - {config['hosts']}")

        return client

    @AEROSPIKE_WRITE.time()
    def put_message(self, aerospike_set, aerospike_key, aerospike_message):
        key = (AerospikeConfig.AEROSPIKE_NAMESPACE, aerospike_set, aerospike_key)
        try:
            logger.info(msg=f"Add message to Aerospike: KEY - {key}")
            self.client.put(key, aerospike_message)
        except Exception as e:
            logger.error(msg=f"error: {e}")

    @AEROSPIKE_READ.time()
    def read_message(self, aerospike_set, aerospike_key):
        try:
            logger.info(msg=f'Read data from Aerospike. KEY - {aerospike_key}, SET - {aerospike_set}')
            key = (AerospikeConfig.AEROSPIKE_NAMESPACE, aerospike_set, aerospike_key)
            (key, metadata, record) = self.client.get(key)
        except Exception as err:
            logger.error(msg=f"Can`t read data from Aerospike. Return empty list of notifications for key - {aerospike_key}\nError: {err}")
            return []

        return record

    @AEROSPIKE_SCAN.time()
    def scan_keys(self, aerospike_set):
        s = self.client.scan(AerospikeConfig.AEROSPIKE_NAMESPACE, aerospike_set)
        records = []

        # callback to be called for each record read
        def callback(input_tuple):
            (_, _, record) = input_tuple
            records.append(record)
            return records

        # invoke the operations, and for each record invoke the callback
        s.foreach(callback)

        return records


# aerospike_client = AerospikeClient()
# aerospike_client.scan_keys('aggr_notifications')
# aerospike_client.put_message('aerospike_set', 'my_key', {'name': 'aer', 'test': 2, 'olol': 12})
# print(json.dumps(aerospike_client.read_message('aggr_notifications', '1000___voice___active___notifications')))




import logging
from microservice_template_core.settings import LoggerConfig, ServiceConfig
import logging_loki
from multiprocessing import Queue

logger = None


def get_logger():
    global logger
    if not logger:
        logger = logging.getLogger(ServiceConfig.SERVICE_NAME)
        logger.setLevel(LoggerConfig.LOG_LEVEL)
        log_format = logging.Formatter('[%(levelname)s] - %(message)s')

        loki_handler = logging_loki.LokiQueueHandler(
            Queue(-1),
            url=f"http://{LoggerConfig.LOKI_SERVER}:{LoggerConfig.LOKI_PORT}/loki/api/v1/push",
            tags={"service": ServiceConfig.SERVICE_NAME, "namespace": ServiceConfig.SERVICE_NAMESPACE},
            version="1",
        )
        loki_handler.setFormatter(log_format)

        console_handler = logging.StreamHandler()

        logger.addHandler(loki_handler)
        logger.addHandler(console_handler)

    return logger




from functools import wraps
from flask import request
from flask_jwt_extended import decode_token
from flask_jwt_extended.jwt_manager import ExpiredSignatureError, InvalidTokenError
from microservice_template_core.tools.logger import get_logger

logger = get_logger()


def token_required():
    def wrapper(fn):
        @wraps(fn)
        def decorator(*args, **kwargs):
            # Make endpoint in the Auth Service to validate an Auth Token
            # The endpoint will return details such as User's Account ID

            invalid_msg = {
                'message': 'Invalid token. Registration and / or authentication required',
                'authenticated': False
            }

            expired_msg = {
                'message': 'Expired token. Re authentication required',
                'authenticated': False
            }

            absent_token = {
                'message': 'AuthToken is not present in header of request',
                'authenticated': False
            }

            auth_token = request.headers.get('AuthToken')
            if auth_token is None:
                return absent_token, 401
            try:
                data = decode_token(auth_token, allow_expired=False)
                if data:
                    return fn(*args, **kwargs)
                else:
                    # You can also redirect the user to the login page.
                    logger.error(
                        msg=f"User auth was failed\nMessage: {invalid_msg}\nHeader: {request.headers}"
                    )
                    return invalid_msg, 401

            except ExpiredSignatureError as err:
                logger.error(
                    msg=f"User auth was failed\nMessage: {expired_msg}\nHeader: {request.headers}\nError: {err}"
                )
                return expired_msg, 401
            except (InvalidTokenError, Exception) as err:
                logger.error(
                    msg=f"User auth was failed\nMessage: {invalid_msg}\nHeader: {request.headers}\nError: {err}"
                )
                return invalid_msg, 401
        return decorator
    return wrapper
